# Copyright 2022 Le Filament (https://le-filament.com)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html)

import logging

from odoo import http
from odoo.http import request

from odoo.addons.portal.controllers.portal import CustomerPortal

_logger = logging.getLogger(__name__)


class CustomerPortal(CustomerPortal):

    # ------------------------------------------------------
    # Inherit Parent
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Common functions
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Routes
    # ------------------------------------------------------
    @http.route(["/releases"], type="http", auth="public", website=True)
    def portal_release_list(self, partner, access_token=None, **kw):
        """
        Route qui affiche les release du client
        """
        values = {}
        partner_sudo = request.env["res.partner"].sudo().browse(int(partner))
        if not partner_sudo:
            _logger.warning("Access denied to release page: no partner found")
            return request.redirect("/")
        if not access_token or access_token != partner_sudo.access_token:
            _logger.warning("Access denied to partner %s release" % partner_sudo.name)
            return request.redirect("/")

        values["release_ids"] = partner_sudo.release_ids
        values["company"] = partner_sudo.name

        return request.render(
            "lefilament_release.partner_release_list_portal_template", values
        )

    @http.route(
        ["/releases/<int:partner_id>/<int:release_id>"],
        type="http",
        auth="public",
        website=True,
    )
    def portal_releases(self, partner_id, release_id, access_token=None, **kw):
        """
        Route permettant de lister l'ensemble des bons de commandes
        liés à l'utilisateur
        """
        values = {}
        partner_sudo = request.env["res.partner"].sudo().browse(partner_id)
        release_sudo = request.env["res.partner.release"].sudo().browse(release_id)
        if not release_sudo or not partner_sudo:
            _logger.warning(
                "Access denied to release page: no partner or release found"
            )
            return request.redirect("/")
        if not access_token or access_token != release_sudo.access_token:
            _logger.warning("Access denied to release %s" % release_sudo.name)
            return request.redirect("/")

        values["home_link"] = partner_sudo.release_share_link
        values["release"] = release_sudo

        return request.render(
            "lefilament_release.partner_release_detail_template", values
        )
