# © 2022 Le Filament (<http://www.le-filament.com>)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html).

import requests

from odoo import _, exceptions, fields, models


class ResPartner(models.Model):
    _name = "res.partner"
    _inherit = ["portal.mixin", "res.partner"]

    # ------------------------------------------------------
    # Fields declaration
    # ------------------------------------------------------
    release_ids = fields.One2many(
        comodel_name="res.partner.release", inverse_name="partner_id", string="Releases"
    )
    module_ids = fields.Many2many(
        comodel_name="res.partner.module",
        column1="partner_id",
        column2="module_id",
        string="Modules",
    )
    release_share_link = fields.Char(
        string="Lien de partage client",
        compute="_compute_release_share_link",
    )
    server_url = fields.Char("URL du serveur")
    database_name = fields.Char("Nom de la base de données")

    # ------------------------------------------------------
    # Computed fields / Search Fields
    # ------------------------------------------------------
    def _compute_release_share_link(self):
        for partner in self:
            if partner.access_token:
                partner.release_share_link = (
                    partner.get_base_url()
                    + "/releases?partner="
                    + str(partner.id)
                    + "&access_token="
                    + partner.access_token
                )
            else:
                partner.release_share_link = ""

    # ------------------------------------------------------
    # Actions
    # ------------------------------------------------------
    def set_release_url(self):
        for partner in self:
            if partner.is_company:
                partner._portal_ensure_token()

    def retrieve_modules(self):
        existing_modules = self.env["res.partner.module"].search([])
        for partner in self:
            if not partner.server_url or not partner.database_name:
                raise exceptions.UserError(
                    _("L'url et la base de données doivent être renseignés.")
                )

            try:
                response = requests.get(
                    partner.server_url + "/get-modules",
                    headers={"DbName": partner.database_name},
                ).json()
            except Exception as e:
                raise exceptions.UserError(e.__str__())

            for module in response:
                if module.get("author_type") != "odoo":
                    # Mise à jour du module
                    if module.get("name") in existing_modules.mapped("name"):
                        existing_modules.filtered(
                            lambda m: m.name == module.get("name")
                        ).update(
                            {
                                "author": module.get("author"),
                                "author_type": module.get("author_type"),
                                "latest_version": module.get("latest_version"),
                                "partner_ids": [(4, partner.id, 0)],
                            }
                        )
                    else:
                        # Création du module
                        partner.update({"module_ids": [(0, 0, module)]})
            # Check modules désinstallés
            module_uninstalled = list(
                set(partner.module_ids.mapped("name"))
                - set(
                    map(
                        lambda m: m.get("name")
                        if m.get("author_type") != "odoo"
                        else None,
                        response,
                    )
                )
            )
            if module_uninstalled:
                module_ids = partner.module_ids.filtered(
                    lambda m: m.name in module_uninstalled
                )
                for module in module_ids:
                    partner.update({"module_ids": [(3, module.id, 0)]})

    # ------------------------------------------------------
    # Common function
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Portal methods
    # ------------------------------------------------------
